%{
Paper:   "Aggregate Dynamics in Lumpy Economies"
Authors:  Isaac Baley and Andres Blanco
Date:     04/12/2020

Code:   Generates Table II in the main text.
        Solves the Bernoulli fixed-cost model for 3 configurations and computes implied moments and CIR_1.
           a) Pure state-dependent (Bernoulli with lambda=0)
           b) Pure time-dependent  (Bernoulli with lambda>0, theta^+ = infinity, theta^- = infinity)
           c) Extended Bernoulli   (Bernoulli with lambda^+ != lambda^-)
        All models assume the stochastic process for uncontrolled capital gaps recovered from the data.
        All models match expected duration of inaction E[\tau] from the data.
%}

clear;
close all;
clc;

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%               Common parameters and options
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Parameters and moments from the data
p.nu          = 0.09518;                                                   % drift: \nu
p.sigma       = 0.2226;                                                    % volatility: \sigma
p.tnu         = p.nu/p.sigma^2;                                            % normalized drift: \nu/\sigma^2
data.Etau     = 2.5188;                                                    % expected duration of inaction: E[\tau]
data.x_hat    = 0.0045;                                                    % reset capital gap: x^{*}
data.Var      = 0.1245;                                                    % variance of gaps: Var[x]
data.Cov      = 0.5998;                                                    % covariance of gaps and age: Cov[x,a]

% Pre-assigned parameters
p.r           = 0.05;                                                      % discount factor
p.alpha       = 0.21/(1-0.64);                                             % capital-share in the production function
p.rho         = p.r -0.02-p.sigma^2/2;                                     % firm discounting

% Options
p.print       = 0;                                                         % print statistics
p.Mx_max      = 3;                                                         % maximum number of moments to compute
p.check       = 0;                                                         % check calculation
p.tol         = 0.0000001;                                                 % tolerance of numerical error
p.N           = 4000;                                                      % size grid in finite differences
p.num_der     = 0;                                                         % size grid in finite differences
p.eps_der     = 1e-8;

p.options     = optimoptions('fsolve','Display','iter','FunctionTolerance',1e-14,'OptimalityTolerance',1e-7);
p.options_opt = optimset('Algorithm','active-set','UseParallel',false,'Display', 'off');


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Solve model and compute ergodic moments for 3 different configurations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%{
For each model configuration:

 - Step1: Solves for optimal policy in the space of (non-centered) gaps:
                  (x_hat^{-},x_hat^{\ast},x_hat^{+})
          Uses finite-difference method. See Lemma 1 for details.

 - Step2: Expresses optimal policy in the space of centered gaps
          From equation 21:
          (x^{-},x^{\ast},x^{+}) = (x_hat^{-}-E[x_hat],x_hat^{\ast}-E[x_hat],x_hat^{+}-E[x_hat])

 - Step3: Computes moments (Var[x], Cov[x,a]) using finite differences.
          Computes CIR_1 = (Var[x]+nu Cov[x,a])/sigma^2 from Corollary 1.

-  Step4: Computes physical cost of adjusment (see footnote 28).

The parameters are calibrated to match expected duration E[\tau] from data.

%}

% Initialize vectors to save results for 3 models

%in terms of non-centered gaps
x_hat_lb  = NaN(1,3);                                                      %lower inaction threshold
x_hat_ast = NaN(1,3);                                                      %reset point
x_hat_ub  = NaN(1,3);                                                      %upper inaction threshold
E_x_hat   = NaN(1,3);                                                      %average

%in terms of centered gaps
x_lb      = NaN(1,3);                                                      %lower inaction threshold (centered)
x_ast     = NaN(1,3);                                                      %reset point (centered)
x_ub      = NaN(1,3);                                                      %upper inaction threshold (centered)

%steady-state moments:
Mx        = ones(1,p.Mx_max);                                              %aux
Max       = ones(1,p.Mx_max);                                              %aux
Etau      = NaN(1,3);                                                      %expected duration of inaction
V_x       = NaN(1,3);                                                      %variance of gaps
Cov_ax    = NaN(1,3);                                                      %covariance of gaps and age
CIR       = NaN(1,3);                                                      %cumulative impulse response for mean (CIR_1)

%other moments and objects:
Epi       = NaN(1,3);                                                      %expected revenue during a period of inaction
freq_ub   = NaN(1,3);                                                      %relative frequency of hitting the upper threshold
freq_lb   = NaN(1,3);                                                      %relative frequency of hitting the lower threshold
phy_cost  = NaN(1,3);                                                      %physical adjustment cost as % of yearly revenue


% Table of results for the paper
TABLE_PAPER    = NaN(10,4);

%% Solves 3 model configurations. Output = ergodic moments, reset point and CIR
for mm = 1:3
    
    if mm==1 % MODEL 1: Bernoulli (lambda = 0, Column II)
        
        p.lambda   = 0.00001;
        p.theta_lb = 0.04293 ;
        p.theta_ub = 2182;                                                 %infinity
        
        TABLE_PAPER(1,2) = p.theta_lb;
        
        x_hat_0    = [4 , -1,1.3];                                         % initial condition for policy
        model_type = 'Fin_diff_Ber';                                       % harzard type
        
        
    elseif mm==2  % MODEL 2:  Bernoulli (lambda >0, Column III)
        
        p.lambda   = 0.397;
        p.theta_lb = 13.113;                                               %infinity
        p.theta_ub = 740.7457;                                             %infinity
        
        TABLE_PAPER(3,3)=p.lambda;
        TABLE_PAPER(4,3)=p.lambda;
        
        
        x_hat_0    = [0.98,1.39,1.38];                                     % initial condition for policy
        model_type = 'Fin_diff_Ber';                                       % harzard type
        
    elseif mm==3 % MODEL 3: Extended Bernoulli (lambda^->0, lambda^+=0, Column IV)
        
        p.lambda   = 0.800;
        p.theta_lb = 0.94549;
        p.theta_ub = 3290;                                                 %infinity
        
        TABLE_PAPER(1,4)=p.theta_lb;
        TABLE_PAPER(3,4)=p.lambda;
        
        
        x_hat_0    = [3.1500,0.49267,0.91833];                             % initial condition for policy
        model_type = 'Fin_diff_ExtBer';                                    % harzard type
        
    end
    
    % Define parameters and roots
    p.tlambda     = p.lambda/p.sigma^2;
    p.xi1         = p.tnu-(p.tnu.^2+2*p.tlambda).^0.5; % Root 1
    p.xi2         = p.tnu+(p.tnu.^2+2*p.tlambda).^0.5; % Root 2
    
    % Step 1: Solve for policy (calls value_function.m)
    
    [X,~, exit_f] = fsolve(@(x_hat_0)value_function(x_hat_0,p,model_type),x_hat_0,p.options);
    [~,SS]        = value_function(X,p,model_type);
    
    x_hat_lb(mm)  = SS.x_hat_lb;
    x_hat_ast(mm) = SS.x_hat_ast;
    x_hat_ub(mm)  = SS.x_hat_ub;
    
    % Compute average yearly payment of adjustment costs relative to revenue
    Etau(mm)      = inaction_moments(0,p,model_type,p.lambda,SS,'expectation');
    Epi(mm)       = inaction_moments(1,p,model_type,p.lambda,SS,'profits');
    SS.Etau       = Etau(mm);
    freq_ub(mm)   = inaction_moments(1,p,model_type,p.lambda,SS,'freq_up');
    freq_lb(mm)   = inaction_moments(1,p,model_type,p.lambda,SS,'freq_down');
    phy_cost(mm)  = (p.theta_lb*freq_lb(mm) + p.theta_ub*freq_ub(mm))/(Epi(mm))*100;
    
    % Step 2: Center policy around steady-state average
    E_x_hat(mm)   = inaction_moments(1,p,model_type,p.lambda,SS,'expectation');  %
    SS.x_lb       = SS.x_hat_lb -E_x_hat(mm)./Etau(mm);
    SS.x_ast      = SS.x_hat_ast-E_x_hat(mm)./Etau(mm);
    SS.x_ub       = SS.x_hat_ub -E_x_hat(mm)./Etau(mm);
    
    x_lb(mm)      = SS.x_lb;
    x_ast(mm)     = SS.x_ast;
    x_ub(mm)      = SS.x_ub;
    
    % Step 3: Ergodic moments and CIR
    
    for nn=1:p.Mx_max
        
        if nn==1  %computes V[x] and Cov[x,a]
            
            [Mx(1,nn),A,vx0,x_grid]   = ergodic_moments(nn-1,p,model_type,SS);
             Max(1,nn)                = compute_covariance(A,vx0,x_grid,SS);
            
        elseif nn>=2
            
            [Mx(1,nn),A,vx,x_grid]    = ergodic_moments(nn-1,p,model_type,SS);
             Max(1,nn)                = compute_covariance(A,vx,x_grid,SS);
            
            if nn==2  %computes CIR_1
                
                % transition matrix for the CIR
                indices        = (1:1:p.N)';
                indices_d      = [1;indices(1:end-1)];
                B              =    sparse(indices  ,indices_d, -1/2,p.N,p.N);
                B        	   = B +sparse(indices_d,indices  ,  1/2,p.N,p.N);
                B(1,:)         = 0;
                B(end,:)       = 0;
                B(1,1)         = 1;
                B(end,end)     = 1;
                CIR(mm)        = compute_CIR(A,vx,x_grid,B,vx0,Mx(1,nn),SS);
                
            end
        end
        
    end
    
    % moments for the CIR
    V_x(mm)    = Mx(3);
    Cov_ax(mm) = Max(2)-bsxfun(@times,Mx(2),Max(1));
    
end

% Colum I: Data
TABLE_PAPER(5,1) = data.Etau;
TABLE_PAPER(6,1) = data.Var;
TABLE_PAPER(7,1) = data.Cov;
TABLE_PAPER(8,1) = data.x_hat;
TABLE_PAPER(9,1) = 3.663;

% Colum II-III-IV: 3 models
TABLE_PAPER(5 ,2:4) = Etau;
TABLE_PAPER(6 ,2:4) = V_x;
TABLE_PAPER(7 ,2:4) = Cov_ax;
TABLE_PAPER(8 ,2:4) = x_ast;
TABLE_PAPER(9 ,2:4) = CIR;
TABLE_PAPER(10,2:4) = phy_cost;

% Print table
TABLE_PAPER