%{
Paper:  Aggregate Dynamics in Lumpy Economies 
Author: Isaac Baley and Andres Blanco 
Date:   04/10/2020 

Code:   Generates firm's optimality conditions as a function mooth pasting
and value matching conditions for the Beernoulli investment model
Notes:   
 

%}

function [FF,SS]=value_function(x,p,model_type)

% Ss bands and menu cost

SS.x_hat_lb   = x(1);
SS.x_hat_ast  = SS.x_hat_lb +exp(x(2));
SS.x_hat_ub   = SS.x_hat_ast+exp(x(3));

 
% Grid for finite difference

x_grid   =linspace(SS.x_hat_lb ,SS.x_hat_ub,p.N)';
Deltah   =x_grid(2)-x_grid(1);

% Index for optimal reset capital

indices        =(1:1:p.N)';
indices_d      =[1;indices(1:end-1)];
[~,ind_ast]    =min(abs(x_grid-SS.x_hat_ast));

% Generate the finite difference hazard

switch  model_type

	case 'Fin_diff_Ber'

    	LAMDA =repmat(p.lambda,p.N,1);

	case 'Fin_diff_ExtBer'

    	LAMDA =p.lambda*(x_grid<SS.x_hat_ast);     

end

% Transition probability: HBJ
A              = sparse(indices,indices, p.sigma^2/Deltah^2+LAMDA+p.rho,p.N,p.N);
A              = A +sparse(indices,indices_d,-p.nu/Deltah/2-p.sigma^2/Deltah^2/2,p.N,p.N);
A        	   = A +sparse(indices_d,indices, p.nu/Deltah/2-p.sigma^2/Deltah^2/2,p.N,p.N);

% Apply Lagrange Interpolation formula for v(x_hat_ast) in the grid

x_1            =x_grid(ind_ast-1);
x_2            =x_grid(ind_ast);
x_3            =x_grid(ind_ast+1);

w1             =(SS.x_hat_ast-x_2)*(SS.x_hat_ast-x_3)/(x_1-x_2)/(x_1-x_3);
w2             =(SS.x_hat_ast-x_1)*(SS.x_hat_ast-x_3)/(x_2-x_1)/(x_2-x_3);
w3             =(SS.x_hat_ast-x_1)*(SS.x_hat_ast-x_2)/(x_3-x_1)/(x_3-x_2);

A(:,ind_ast-1) =A(:,ind_ast-1)-w1*LAMDA;
A(:,ind_ast  ) =A(:,ind_ast  )-w2*LAMDA;
A(:,ind_ast+1) =A(:,ind_ast+1)-w3*LAMDA;

% Flow profits in the interior of the continuation region
f              =exp(p.alpha*x_grid)+LAMDA.*(exp(x_grid)-exp(SS.x_hat_ast));

% Value matching transition probabilities
A(1,:)           =0;
A(p.N,:)         =0;

A(1,ind_ast-1)   =w1;
A(1,ind_ast  )   =w2;
A(1,ind_ast+1)   =w3;
A(1,1)           =-1;

A(p.N,ind_ast-1) =(SS.x_hat_ast-x_2)*(SS.x_hat_ast-x_3)/(x_1-x_2)/(x_1-x_3);
A(p.N,ind_ast  ) =(SS.x_hat_ast-x_1)*(SS.x_hat_ast-x_3)/(x_2-x_1)/(x_2-x_3);
A(p.N,ind_ast+1) =(SS.x_hat_ast-x_1)*(SS.x_hat_ast-x_2)/(x_3-x_1)/(x_3-x_2);
A(p.N,p.N)       =-1;

% Value matching flow period           
f(1)             =exp(SS.x_hat_ast)+p.theta_lb-exp(x_grid(1));
f(p.N)           =exp(SS.x_hat_ast)+p.theta_ub-exp(x_grid(p.N));

% Value function given the Ss bands            
val_0          =A\f; 

f_1            =val_0(ind_ast-1);
f_2            =val_0(ind_ast);
f_3            =val_0(ind_ast+1);


FF             = ones(2,1);    

% Value matching in the lower Ss bands
FF(1)          = ((val_0(2)-val_0(1))/Deltah-exp(SS.x_hat_lb))./exp(SS.x_hat_lb);

% Apply Lagrange Interpolation formula for v'(x_hat_ast) in the grid
df_1            =(val_0(ind_ast)-val_0(ind_ast-2))/Deltah/2;
df_2            =(val_0(ind_ast+1)-val_0(ind_ast-1))/Deltah/2;
df_3            =(val_0(ind_ast+2)-val_0(ind_ast))/Deltah/2;
FF(2)          = (w1*df_1+w2*df_2+w3*df_3-exp(SS.x_hat_ast))./exp(SS.x_hat_ast);

% Value matching in the upper Ss bands
FF(3)          = ((val_0(p.N)-val_0(p.N-1))/Deltah-exp(SS.x_hat_ub))./exp(SS.x_hat_ub);


if p.check ==1 

	F_check      =ones(3,1);
    F_check(1)   = w1*f_1+w2*f_2+w3*f_3-exp(SS.x_hat_ast)-p.theta_lb-(val_0(1)  -exp(x_grid(1) ));   
    F_check(3)   = w1*f_1+w2*f_2+w3*f_3-exp(SS.x_hat_ast)-p.theta_ub-(val_0(p.N)-exp(x_grid(p.N)));
 
    aux_vax      =-p.rho*val_0(2:(p.N-1))+exp(p.alpha*x_grid(2:(p.N-1)))-p.nu*(val_0(3:(p.N))-val_0(1:(p.N-2)))/Deltah/2;
    aux_vax      =aux_vax+p.sigma^2/2*(val_0(3:(p.N))-2*val_0(2:(p.N-1))+val_0(1:(p.N-2)))/Deltah^2;
    aux_vax      =aux_vax+LAMDA(2:(p.N-1)).*(w1*f_1+w2*f_2+w3*f_3-val_0(2:(p.N-1))-exp(SS.x_hat_ast)+exp(x_grid(2:(p.N-1))));
    F_check(2)   = norm(aux_vax);

    fprintf('check value function \n');
    fprintf('error in HJB             : %6.2e \n', norm(F_check(2))); 
    fprintf('error in value matching  : %6.2e \n', norm(F_check(1))+norm(F_check(3)));
    fprintf('smooth pasting lb        : %6.2f \n', norm(FF(1)));
    fprintf('optimality reset         : %6.2f \n', norm(FF(2)));
    fprintf('smooth pasting ub        : %6.2f \n', norm(FF(3)));
  
end

if nargout>=3

    SS.k_grid =x_grid;

end

return
 